#!/bin/bash
#
# Caddy Service

PATH=$HOME/.docker/cli-plugins:/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

config_caddy() {
  echo -ne "\n* Configuring /federated/apps/caddy container.."
  spin &
  SPINPID=$!

  if [ ! -d "/federated/apps/caddy" ]; then
    mkdir -p /federated/apps/caddy/data/data
    mkdir -p /federated/apps/caddy/data/srv
    mkdir -p /federated/apps/caddy/data/etc/caddy
  fi

  DOMAIN_ARRAY=(${DOMAIN//./ })
  DOMAIN_FIRST=${DOMAIN_ARRAY[0]}
  DOMAIN_LAST=${DOMAIN_ARRAY[1]}

cat > /federated/apps/caddy/docker-compose.yml <<EOF
version: "3.7"

services:
  caddy:
    image: federatedcomputer/caddy:\${IMAGE_VERSION}
    container_name: www
    hostname: www.$DOMAIN
    domainname: $DOMAIN
    restart: always
    networks:
      federated:
        ipv4_address: 172.99.0.31
    extra_hosts:
      - "gitea.$DOMAIN:$EXTERNALIP"
    env_file:
      - ./.env
    volumes:
      - ./data/srv:/srv
      - ./data/etc/caddy/Caddyfile:/etc/caddy/Caddyfile
      - ./data/data:/data

networks:
  federated:
    external: true
EOF

cat > /federated/apps/caddy/.env <<EOF
IMAGE_VERSION="latest"
VIRTUAL_PROTO=http
VIRTUAL_PORT=80
VIRTUAL_HOST=www.$DOMAIN,blog.$DOMAIN,documentation.$DOMAIN
EOF
chmod 600 /federated/apps/caddy/.env

cat > /federated/apps/caddy/data/etc/caddy/Caddyfile <<EOF
www.$DOMAIN:80 {
  root * /srv/www.$DOMAIN/public
  file_server
  route /webhook {
    webhook {
      repo https://gitea.$DOMAIN/gitea/www.$DOMAIN
      type github
      secret $WEBHOOK_SECRET
      submodule
      token $GITEA_TOKEN_1
      path www.$DOMAIN
      branch master
    }
  }
}
blog.$DOMAIN:80 {
  root * /srv/blog.$DOMAIN/public
  file_server
  route /webhook {
    webhook {
      repo https://gitea.$DOMAIN/gitea/blog.$DOMAIN
      type github
      secret $WEBHOOK_SECRET
      submodule
      token $GITEA_TOKEN_1
      path blog.$DOMAIN
      branch master
    }
  }
}
documentation.$DOMAIN:80 {
  root * /srv/documentation.$DOMAIN/public
  file_server
  route /webhook {
    webhook {
      repo https://gitea.$DOMAIN/gitea/documentation.$DOMAIN
      type github
      secret $WEBHOOK_SECRET
      submodule
      token $GITEA_TOKEN_1
      path documentation.$DOMAIN
      branch master
    }
  }
}
EOF

kill -9 $SPINPID &> /dev/null
echo -ne "done."
}
start_caddy() {
  # Start service with command to make sure it's up before proceeding
  start_service "caddy" "nc -z 172.99.0.31 80 &> /dev/null"

  kill -9 $SPINPID &> /dev/null
  echo -ne "done."
}
